/*
 * UsuarioSessionBean.java
 *
 * Created on 30 de Julho de 2006, 00:31
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package info.everest.session.seguranca;

import info.everest.entity.Usuario;
import info.everest.session.SessionException;
import java.util.List;
import javax.annotation.Resource;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceUnit;
import javax.transaction.UserTransaction;

/**
 *
 * @author Richter
 */
@Stateless
public class UsuarioSessionBean implements UsuarioSessionRemote {

  @PersistenceUnit(name="Everest-PU")
  private EntityManagerFactory emf;

  @Resource()
  UserTransaction tx;

  /** Creates a new instance of UsuarioSessionBean */
  public UsuarioSessionBean() {
  }
  
  @SuppressWarnings(value={"unchecked"})
  public List<Usuario> getAll() throws SessionException {
    EntityManager em = getEM();
    try {
      return (List<Usuario>)(em.createQuery("select object(u) from Usuario u order by u.usuario").getResultList());
    } catch(Exception ex) {
      throw new SessionException(ex);
    } finally {
      em.close();
    }
  }

  public boolean usuarioValido(String usuario, String senha) throws SessionException {
    // Enviar a senha como parametro pode ser uma falha de
    // segurana, pois um programa monitor de SQL como
    // o SQL Server Profiler ir mostrar as senhas.
    Usuario u = getUsuario(usuario);

    // alguns bancos de dados so case insensitive
    // ento, tem que carregar e ainda assim, comparar o usuario e a senha.
    // s para garantir que obedece a regra das
    // maisculas e minsculas!!!
    return (u!=null
            && u.getUsuario().equals(usuario)
            && u.getSenha().equals(senha));
  }

  @SuppressWarnings(value={"unchecked"})
  public Usuario getUsuario(String usuario)
         throws SessionException {
    EntityManager em = getEM();
    try {
      List<Usuario> users = (List<Usuario>)em.createNamedQuery("Usuario.findByUsuario").setParameter("usuario", usuario).getResultList();
      return (users!=null && users.size()==1?users.get(0):null);
    } catch(Exception ex) {
      throw new SessionException(ex);
    } finally {
      em.close();
    }
  }
  
  public boolean usuarioHabilitado(String usuario, String role) throws SessionException {
    return false;
  }
  
  public boolean trocarSenha(String usuario, String senhaAntiga, String senhaNova, String confirmaSenha)
         throws SessionException {
    if(usuario==null) {
      throw new SessionException("Usurio  obrigatrio.");
    }

    Usuario u = getUsuario(usuario);
    
    if(u==null) {
      throw new SessionException("Usurio "+usuario+" no localizado.");
    }
    
    if(u.getSenha()!=null && !u.getSenha().equals(senhaAntiga)) {
      throw new SessionException("Senha antiga no confere.");
    }
    
    if(senhaNova==null || confirmaSenha==null || !senhaNova.equals(confirmaSenha)) {
      throw new SessionException("Senha nova e confirmao da senha no conferem.");
    }

    EntityManager em = getEM();
    
    try {
      em.merge(u);
      u.setSenha(senhaNova);
      return true;
    } catch(Exception e) {
      throw new SessionException(e);
    } finally {
      em.close();
    }
  }
  
  private EntityManager getEM() {
    return emf.createEntityManager();
  }
}
